/*****************************************************************************/
// ssgm.c
// SSGM control
// 2016/10/27
// Hyunsik Yi
/*****************************************************************************/

/*****************************************************************************/
// header files
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <memory.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <pthread.h>
#include <semaphore.h>

#include "ssgm_ioctl.h"
#include "ssgm.h"

#define PULSE_ON 1

/*****************************************************************************/
// global variables
/*****************************************************************************/
int dh = 0;	// FD for SSGM device


/*****************************************************************************/
// function implementation
/*****************************************************************************/
char getch()
{
	char buf = 0;
	struct termios old = {0};

	if( tcgetattr(0, &old) < 0) perror("tcsetattr()");
	old.c_lflag &= ~ICANON;
	old.c_lflag &= ~ECHO;
	old.c_cc[VMIN] = 1;
	old.c_cc[VTIME] = 0;

	if( tcsetattr(0, TCSANOW, &old) < 0) perror("tcsetattr()");
	if( read(0, &buf, 1) < 0) perror("read()");
	old.c_lflag |= ICANON;
	old.c_lflag |= ECHO;

	if(tcsetattr(0, TCSADRAIN, &old) < 0) perror("tcsetattr()");

	return(buf);
}


/*****************************************************************************/
int ssgmOpenDevice()
{
	dh = open( SSGM_DEVICE_NAME, O_RDWR | O_SYNC);
	if(dh < 0)
	{
		printf( "dev file(%s) open error(err no.: %d).\n", SSGM_DEVICE_NAME, dh ); 
  		perror("open_err");
  		return -1;
	}

	return 0;
}


/*****************************************************************************/
void ssgmCloseDevice()
{
	close(dh);
}


/*****************************************************************************/
int ssgmGetReg(SSGM_PARAM *param)
{
	if( ioctl(dh, IOCTL_GET_SSGM_REG, param) < 0)
	{
		printf( "ioctl error...dh:%d, param-addr: %d, param-val: %d.\n", 
			dh, param->addr, param->val ); 
		return -1;
	}
	return 0;
}


/*****************************************************************************/
int ssgmSetReg(SSGM_PARAM *param)
{
	if( ioctl(dh, IOCTL_SET_SSGM_REG, param) < 0)  return -1;
	return 0;
}


/*****************************************************************************/
int ssgmGetRevision(unsigned int* val)
{
	SSGM_PARAM ssgmParam;
	ssgmParam.addr = ADDR_PROD_REV;

	if( ssgmGetReg(&ssgmParam) )  return -1;
	
	*val = ssgmParam.val;

	return 0;
}


/*****************************************************************************/
int ssgmGetBuildDate(unsigned int* val)
{
	SSGM_PARAM ssgmParam;
	ssgmParam.addr = ADDR_BLD_DATE;

	if( ssgmGetReg(&ssgmParam))  return -1;
	
	*val = ssgmParam.val;

	return 0;
}

/*****************************************************************************/
int ssgmGetSerialNum(unsigned int* val)
{
	SSGM_PARAM ssgmParam;
	ssgmParam.addr = ADDR_SRL_NUM;

	if(ssgmGetReg(&ssgmParam))  return -1;
	
	*val = ssgmParam.val;

	return 0;
}

/*****************************************************************************/
int ssgmGetControlReg(unsigned int* val)
{
	SSGM_PARAM ssgmParam;
	ssgmParam.addr = ADDR_CTRL_REG;

	if(ssgmGetReg(&ssgmParam))  return -1;
	
	*val = ssgmParam.val;

	return 0;
}

/*****************************************************************************/
int ssgmGetPulseCount(int pulse_type, unsigned int* val)
{
	SSGM_PARAM ssgmParam;

	if((pulse_type < MCP) || (pulse_type > SOP4))
	{
		printf("[%s] ... ERROR (illegal pulse type: %d\n", __FUNCTION__, pulse_type);
		return -1;
	}

	ssgmParam.addr = (ADDR_MCP_CNT + 0x04*pulse_type);

	if(ssgmGetReg(&ssgmParam)) return -1;
	
	*val = ssgmParam.val;

	return 0;
}

/*****************************************************************************/
int ssgmSetControlReg(unsigned int val)
{
	SSGM_PARAM ssgmParam;
	ssgmParam.addr = ADDR_CTRL_REG;
	ssgmParam.val = val;

	return ssgmSetReg(&ssgmParam);
}


/*****************************************************************************/
int ssgmGetPulseDuration(int pulse_type, unsigned int* val)
{
	SSGM_PARAM ssgmParam;

	if((pulse_type < MCP) || (pulse_type > NCP)) 
	{
		*val = 0xFFFFFFFF;
		return -1;
	}
	ssgmParam.addr = ADDR_MCP_DUR + 0x04*pulse_type;

	if(ssgmGetReg(&ssgmParam))
	{
		*val = 0xFFFFFFFF;
		return -1;
	}
	
	*val = ssgmParam.val;

	return 0;
}


/*****************************************************************************/
int ssgmSetPulseDuration(int pulse_type, unsigned int val)
{
	SSGM_PARAM ssgmParam;

	if( (pulse_type < MCP) || (pulse_type > NCP) )  return -1;

	ssgmParam.addr = ADDR_MCP_DUR + 0x04*pulse_type;
	ssgmParam.val = val;

	if( ssgmSetReg(&ssgmParam) )  return -1;

	return 0;
}


/*****************************************************************************/
int ssgmGetPulseEnableSts(unsigned int* val)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_PULSE_EN;

	if( ssgmGetReg(&ssgmParam) ) return -1;
	
	*val = ssgmParam.val;

	return 0;
}


/*****************************************************************************/
int ssgmSetPulseEnableSts(int pulse_type, int onoff)
{
	SSGM_PARAM ssgmParam;

	if((pulse_type < MCP) || (pulse_type > SOP4))
	{
		printf("[%s] ... ERROR (illegal pulse type: %d)\n", __FUNCTION__, pulse_type);
		return -1;
	}

	ssgmParam.addr = ADDR_PULSE_EN;

	if( ssgmGetReg(&ssgmParam) ) return -1;
	
	if(onoff == PULSE_ON)	ssgmParam.val |= (1 << pulse_type);
	else			ssgmParam.val &= ~(1 << pulse_type);

	return ssgmSetReg(&ssgmParam);
}

/*****************************************************************************/
int ssgmGetPulseOutputSource(unsigned int *pulse_source)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_PULSE_SEL;
	if( ssgmGetReg(&ssgmParam) < 0)  return -1;

	*pulse_source = ssgmParam.val;

	return 0;
}


/*****************************************************************************/
int ssgmSetPulseOutputSource(int pulse_type, int source)
{
	SSGM_PARAM ssgmParam;

	if((pulse_type < MCP) || (pulse_type > SOP4))
	{
		printf("[%s] ... ERROR (illegal pulse type: %d)\n", __FUNCTION__, pulse_type);
		return -1;
	}

	if((source < MCP) || (source > SOP4))
	{
		printf("[%s] ... ERROR (illegal source type: %d)\n", __FUNCTION__,source);
		return -1;
	}

	ssgmParam.addr = ADDR_PULSE_SEL;

	if( ssgmGetReg(&ssgmParam) < 0) return -1;
	
	ssgmParam.val &= ~(0x0F << (4 * pulse_type)); 
	ssgmParam.val |= (source  & 0x07) << (4 * pulse_type);

//	printf("val = %08X\n", ssgmParam.val);

	return ssgmSetReg(&ssgmParam);
}

/*****************************************************************************/
int ssgmGetPulseHighTime(unsigned int *high_time)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_PULSE_HI;
	if(ssgmGetReg(&ssgmParam) < 0)
	{
		*high_time = 0xFFFFFFFF;
		return -1;
	}

	*high_time = ssgmParam.val;

	return 0;
}

/*****************************************************************************/
int ssgmSetPulseHighTime(unsigned int high_time)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_PULSE_HI;
	ssgmParam.val = high_time;

	if(ssgmSetReg(&ssgmParam) < 0) return -1;

	return 0;
}

/*****************************************************************************/
int ssgmGetSopDelayTime(int sop, unsigned int *delay_time)
{
	SSGM_PARAM ssgmParam;

	if((sop < SOP1) || (sop > SOP4)) {
		*delay_time = 0xFFFFFFFF;
		return -1;
	}

	// SOP1 = 2
	// SOP2 = 3
	// SOP3 = 4
	// SOP4 = 5
	ssgmParam.addr = ADDR_SOP1_DLY + 4*(sop-2);

	if(ssgmGetReg(&ssgmParam) < 0) {
		*delay_time = 0xFFFFFFFF;
		return -1;
	}

	*delay_time = ssgmParam.val;

	return 0;
}


/*****************************************************************************/
int ssgmSetSopDelayTime(int sop, unsigned int delay_time)
{
	SSGM_PARAM ssgmParam;

	if( (sop < SOP1) || (sop > SOP4) ) return -1;

	// SOP1 = 2
	// SOP2 = 3
	// SOP3 = 4
	// SOP4 = 5
	ssgmParam.addr = ADDR_SOP1_DLY + 4*(sop-2);
	ssgmParam.val = delay_time;

	if(ssgmSetReg(&ssgmParam) < 0)  return -1;

	return 0;
}

