/*****************************************************************************/
// ssgm_test.c
// SSGM demo application
// 2016/10/27
// Hyunsik Yi. 
// It was modified by Jeongki Kim at 2020.11.10.
/*****************************************************************************/

/*****************************************************************************/
// header files
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/mman.h>
#include <memory.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <pthread.h>
#include <semaphore.h>

#include "ssgm_ioctl.h"
#include "ssgm.h"

/*****************************************************************************/
// MACRO definition
/*****************************************************************************/
#define RESET		1
#define NORMAL_OP	0

#define PULSE_ON 	1
#define PULSE_OFF	0

#define NCP_START 	1
#define NCP_STOP 	0

#define MODE_CONTINOUS 		1 	// NCP continuous mode
#define MODE_SINGLE_TIME 	0	// NCP single-time mode by external command

#define GEN_INTERNAL 	1	// MCP/NCP self-generation
#define GEN_EXTERNAL 	0	// MCP/NCP generation by external source

#define SOP_SRC_MCP	1	// SOP reference signal: MCP
#define SOP_SRC_NCP	0	// SOP reference signal: NCP

#define SIG_SRC_LOC	0	// MCP/NCP source selection: Local
#define SIG_SRC_OPT	1	// MCP/NCP source selection: optics
#define SIG_SRC_LVDS	2	// MCP/NCP source selection: LVDS

#define MCP_NCP_SYNC    1
#define MCP_NCP_NOSYNC  0

#define REF_CLK 	(48)			// 48Mhz reference clock
#define CLK2USEC(X) 	(int)(X / REF_CLK)	// num. of clocks to usec
#define USEC2CLK(X) 	(unsigned int)(X * REF_CLK) // usec to num. of clocks 

#define MCP_DURATION	600
#define NCP_DURATION	600
#define SOP1_DELAY	0
#define SOP2_DELAY	0
#define SOP3_DELAY	0
#define SOP4_DELAY	0
#define NCP_GEN_CNT	0

/*****************************************************************************/
// type definition
/*****************************************************************************/

// SSGM control register parameter
typedef struct tagSsgmCtrl
{
	unsigned int reset;
	unsigned int ncp_mode;
	unsigned int gen_mode;
	unsigned int sop_source;
	unsigned int mcp_ncp_source;
	unsigned int mcp_ncp_sync;
	unsigned int ncp_op;
} SSGM_CTRL;

/*****************************************************************************/
// function prototypes in "main"
/*****************************************************************************/
int showDevInfo();
int showPulseCount();
int showPulseEnableSts();
int allPulseOnOff(int onoff);
int pulseOnOff(int pulse_type, int onoff);
int showPulseOutputSource();
int setPulseOutputSource(int pulse_type, int source_type);
int showPulseDuration(int pulse_type);
int setPulseDuration(int pulse_type, unsigned int usec);
int showPulseHighTime();
int setPulseHighTime(int usec);
int showSopDelayTime(int sop);
int setSopDelayTime(int sop, int usec);
int showSsgmCtrl();
int setSsgmCtrl(SSGM_CTRL ssgmCtrl);
int startNcp();
int stopNcp();
int setNcpGenCount(unsigned int count);
int setNcpDelay(unsigned int delay);

/*****************************************************************************/
// main function
/*****************************************************************************/
int main() 
{
	SSGM_CTRL ssgmCtrl;
//	unsigned int delay = 0;

	printf(">> Opening SSGM Device . . .");
	if( ssgmOpenDevice() == -1 ) {
		printf("SSGM Device Open ERROR.\n");
		exit(1);
	}
	printf("OK\n");

	bzero( &ssgmCtrl, sizeof(SSGM_CTRL));

	ssgmCtrl.reset = RESET;
	ssgmCtrl.ncp_mode = MODE_CONTINOUS;
//	ssgmCtrl.ncp_mode = MODE_SINGLE_TIME;
	ssgmCtrl.gen_mode = GEN_INTERNAL;
//	ssgmCtrl.sop_source = SOP_SRC_MCP;
	ssgmCtrl.sop_source = SOP_SRC_NCP;
	ssgmCtrl.mcp_ncp_source =  SIG_SRC_LOC;
//	ssgmCtrl.mcp_ncp_sync = MCP_NCP_SYNC;

	setSsgmCtrl(ssgmCtrl);

	allPulseOnOff(PULSE_ON);
	setPulseDuration(MCP, MCP_DURATION);	
	setPulseDuration(NCP, NCP_DURATION);
	setSopDelayTime(SOP1, SOP1_DELAY);
	setSopDelayTime(SOP2, SOP2_DELAY);
	setSopDelayTime(SOP3, SOP3_DELAY);
	setSopDelayTime(SOP4, SOP4_DELAY);
//	setNcpDelay(0);

	setNcpGenCount(NCP_GEN_CNT);

	ssgmCtrl.reset = NORMAL_OP;
	setSsgmCtrl(ssgmCtrl);

	ssgmCtrl.ncp_op = NCP_START;
	setSsgmCtrl(ssgmCtrl);

/*
	pulseOnOff(MCP, PULSE_OFF);
	pulseOnOff(NCP, PULSE_OFF);
	pulseOnOff(SOP1, PULSE_OFF);
	pulseOnOff(SOP2, PULSE_OFF);
	pulseOnOff(SOP3, PULSE_ON);
	pulseOnOff(SOP4, PULSE_ON);
	setPulseOutputSource(MCP, MCP);
	setPulseOutputSource(SOP4, SOP4);
	setPulseDuration(MCP, 200);	
	setPulseDuration(NCP, 600);
	setPulseHighTime(1);
	setSopDelayTime(SOP1, 800);
	setSopDelayTime(SOP2, 9000);
	setSopDelayTime(SOP3, 18000);
	setSopDelayTime(SOP4, 28000);

	showDevInfo();		
	showPulseOutputSource();
	showPulseEnableSts();
	showPulseDuration(MCP);
	showPulseDuration(NCP);
	showPulseHighTime();
	showSopDelayTime(SOP1);
	showSopDelayTime(SOP2);
	showSopDelayTime(SOP3);
	showSopDelayTime(SOP4);

	int tmp = 0;
	int tmp1 = 800;
	int tmp2 = 400;
*/	

	showSsgmCtrl();
	showPulseEnableSts();
	printf("press ESC to exit\n");
	printf("press any key to continue\n");
	char szIn = '\0';

#if 1
	while(1) {
		szIn = getch();
		if(szIn == ESC) break;

		printf("-----------------------------------\n");

		ssgmCtrl.ncp_op = NCP_START;
		setSsgmCtrl( ssgmCtrl );
/*
		if(ssgmCtrl.ncp_mode == MODE_SINGLE_TIME) {
			setNcpDelay(delay);
			delay += 10;
			if(delay >= MCP_DURATION) break;
			startNcp();

//			setSsgmCtrl(ssgmCtrl);
		}
		else {
			if( tmp%2 == 0) setPulseDuration(NCP, tmp1);
			else	setPulseDuration(NCP, tmp2);
			tmp++;			
		}
*/
		showPulseCount();

		printf("-----------------------------------\n");
		printf("press ESC to exit\n");
		printf("press any key to continue\n");
	}
#else
	while(1) {
		setPulseDuration(NCP, tmp1);
		usleep(10000);
		setPulseDuration(NCP, tmp2);
		usleep(10000);

		startNcp();
		usleep(5000);
	}
#endif

	ssgmCloseDevice();
 	printf(">> Closing SSGM Device . . . OK\n");

	return 0;
}

/*****************************************************************************/
// It sets NCP Delay (count)
/*****************************************************************************/
int setNcpDelay(unsigned int delay)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_NCP_DLY;
//	ssgmParam.val = delay;

	ssgmParam.val = USEC2CLK(delay);

	if( ssgmSetReg(&ssgmParam) ) return -1;
	return 0;
}

/*****************************************************************************/
// set NCP Generation Count
/*****************************************************************************/
int setNcpGenCount(unsigned int count)
{
	SSGM_PARAM ssgmParam;

	ssgmParam.addr = ADDR_NCP_GEN_CNT;
	ssgmParam.val = count;

	if( ssgmSetReg(&ssgmParam) ) return -1;
	return 0;
}

/*****************************************************************************/
// shows device information
/*****************************************************************************/
int showDevInfo()
{
	unsigned int val = 0;

	if(ssgmGetRevision(&val) < -1) {
		printf(">> Revision: ERROR\n");
		return -1;
	}
	printf(">> Revision: 0x%08X\n", val);

	if(ssgmGetBuildDate(&val) < -1) {
		printf(">> Build date: ERROR\n");
		return -1;
	}
	printf(">> Build date: 0x%08X\n", val);

	if(ssgmGetSerialNum(&val) < -1) {
		printf(">> Serial Num: ERROR\n");
		return -1;
	}
	printf(">> Serial Num: 0x%08X\n", val);

	return 0;
}

/*****************************************************************************/
// shows current MCP/NCP duration in usec
/*****************************************************************************/
int showPulseDuration(int pulse_type)
{
	unsigned int val = 0;
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};

	if((pulse_type < MCP) || (pulse_type > NCP)) {
		printf("[%s]illegal pulse type: %d\n", __FUNCTION__, pulse_type);
		return -1;
	}

	if(ssgmGetPulseDuration(pulse_type, &val) < -1) {
		printf("[%s]reading %s duration.. ERROR\n", __FUNCTION__, pulse_name[pulse_type] );
		return -1;
	}

	printf("%s duration.. %d usec(0x%08X)\n", pulse_name[pulse_type], CLK2USEC(val), val);
	return 0;
}

/*****************************************************************************/
// sets MCP/NCP duration in usec
/*****************************************************************************/
int setPulseDuration(int pulse_type, unsigned int usec)
{
	unsigned int val = 0;
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};

	if( (pulse_type < MCP) || (pulse_type > NCP) ) {
		printf("[%s]illegal pulse type: %d\n", __FUNCTION__, pulse_type);
		return -1;
	}

	val = USEC2CLK(usec);
	if(ssgmSetPulseDuration(pulse_type, val) < -1) {
		printf( "[%s]reading %s duration.. ERROR\n", __FUNCTION__, pulse_name[pulse_type] );
		return -1;
	}
	return 0;
}


/*****************************************************************************/
// It shows current pulse count
/*****************************************************************************/
int showPulseCount()
{
	unsigned int val = 0;
	int pulse_type = MCP;
	int ret = 0;
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};

	for( pulse_type=MCP; pulse_type <= SOP4; pulse_type++ ) {
		val = 0;
		if( ssgmGetPulseCount(pulse_type, &val) < 0 ) {
			printf("Pulse count... ERROR(%s)\n", pulse_name[pulse_type]);
			ret = -1;
		}		
		printf("%s count ...\t %d\n", pulse_name[pulse_type], val);	
	}
	
	return ret;
}

/*****************************************************************************/
// It shows pulse enable status
/*****************************************************************************/
int showPulseEnableSts()
{
	unsigned int val = 0;
	int pulse_type = MCP;
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};


	if( ssgmGetPulseEnableSts(&val) < 0 ) {
		printf("Pulse enable status... ERROR\n");
		return -1;
	}

	printf("Pulse Enable Reg. Value: 0x%08X\n", val);

	for( pulse_type=MCP; pulse_type <= SOP4; pulse_type++ )
	{
		if( (val >> pulse_type) & 1 ) printf("%s ...\t Enabled\n", pulse_name[pulse_type]);
		else			printf("%s ...\t Disabled\n", pulse_name[pulse_type]);
	}

	return 0;
}

/*****************************************************************************/
// make all pulse-enable ON/OFF
/*****************************************************************************/
int allPulseOnOff(int onoff)
{
	int pulse_type = MCP;

	for( pulse_type=MCP; pulse_type <= SOP4; pulse_type++ )

		if( ssgmSetPulseEnableSts(pulse_type, onoff) < 0) return -1;

	return 0;
}

/*****************************************************************************/
// make all pulse-enable ON/OFF
/*****************************************************************************/
int pulseOnOff(int pulse_type, int onoff)
{
	if((pulse_type < MCP) || (pulse_type > SOP4)) return -1;

	return ssgmSetPulseEnableSts(pulse_type, onoff);
}

/*****************************************************************************/
// shows pulse output source
/*****************************************************************************/
int showPulseOutputSource()
{
	unsigned int val = 0;
	int pulse_type = MCP;
	int src_type = MCP;
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};


	if(ssgmGetPulseOutputSource(&val) < 0) {
		printf("Pulse output source. . . . ERROR\n");
		return -1;
	}

	for(pulse_type = MCP; pulse_type <= SOP4; pulse_type++) {
		src_type = (val >> (4*pulse_type)) & 0x07;

		if( (src_type < MCP) || (src_type > SOP4) )
			printf("illegal output src of %s... %d\n", pulse_name[pulse_type], src_type); 
		else
			printf("output src of %s... %s\n", pulse_name[pulse_type], pulse_name[src_type]);
	}
	return 0;
}

/*****************************************************************************/
// sets pulse(pulse_name)'s source(source_name)
/*****************************************************************************/
int setPulseOutputSource(int pulse_type, int source_type)
{

	if( (pulse_type < MCP) || (pulse_type > SOP4) ) return -1;

	if( (source_type < MCP) || (source_type > SOP4) ) return -1;

	return ssgmSetPulseOutputSource(pulse_type, source_type);
}

/*****************************************************************************/
// shows pulse high time in usec
/*****************************************************************************/
int showPulseHighTime()
{
	unsigned int high_time = 0;

	if(ssgmGetPulseHighTime(&high_time) < 0) {
		printf("[%s]reading pulse high time ... ERROR\n", __FUNCTION__);
		return -1;
	}

	printf("pulse high time... %d usec(0x%08X)\n", CLK2USEC(high_time), high_time);	
	return 0;
}

/*****************************************************************************/
// sets pulse high time in usec
/*****************************************************************************/
int setPulseHighTime(int usec)
{
	unsigned int high_time = USEC2CLK(usec);

	if(ssgmSetPulseHighTime(high_time) < 0) {
		printf("[%s]writing pulse high time... ERROR\n", __FUNCTION__);
		return -1;
	}

	return 0;
}

/*****************************************************************************/
// shows SOP delay time in usec
/*****************************************************************************/
int showSopDelayTime(int sop)
{
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};
	unsigned int delay_time = 0;

	if( (sop < SOP1) || (sop > SOP4) ) {
		printf("[%s]illegal SOP type: %d\n", __FUNCTION__, sop);
		return -1;
	}

	if(ssgmGetSopDelayTime(sop, &delay_time) < 0) {
		printf("[%s]reading SOP delay time . . . ERROR\n", __FUNCTION__);
		return -1;
	}

	printf("%s delay time: %d usec(0x%08X)\n", pulse_name[sop], CLK2USEC(delay_time), delay_time);	
	return 0;
}

/*****************************************************************************/
// sets SOP delay time in usec
/*****************************************************************************/
int setSopDelayTime(int sop, int usec)
{
	unsigned int delay_time = USEC2CLK(usec);
	char* pulse_name[6] = {"MCP", "NCP", "SOP1", "SOP2", "SOP3", "SOP4"};

	if( (sop < SOP1) || (sop > SOP4) ) {
		printf("[%s]illegal SOP type: %d\n",  __FUNCTION__, sop);
		return -1;
	}

	if(ssgmSetSopDelayTime(sop, delay_time) < 0) {
		printf("[%s]writing %s delay time... ERROR\n", __FUNCTION__, pulse_name[sop]);
		return -1;
	}
	
	return 0;
}

/*****************************************************************************/
// sets SSGM control parameters
/*****************************************************************************/
int setSsgmCtrl( SSGM_CTRL ssgmCtrl )
{
	unsigned int val = 0;
	int rtn; 

// reset board
//	ssgmSetControlReg(1);

	val |= (ssgmCtrl.reset << 0);	
	val |= (ssgmCtrl.ncp_mode << 1);
	val |= (ssgmCtrl.ncp_op << 2);
	val |= (ssgmCtrl.gen_mode << 8);
	val |= (ssgmCtrl.sop_source << 9);
	val |= (ssgmCtrl.mcp_ncp_sync << 10);
	val |= (ssgmCtrl.mcp_ncp_source << 16);

	printf("Control Reg. Value: 0x%08X \n", val);

	rtn = ssgmSetControlReg(val);
	if( rtn == -1 ) {
		printf( "In %s, SSGM Set Control Register Error...\n", __FUNCTION__ );
		return -1; 
	}
	else return 0; 
}


/*****************************************************************************/
// shows SSGM control parameters
/*****************************************************************************/
int showSsgmCtrl()
{
	unsigned int val = 0;
	SSGM_CTRL ssgmCtrl;

	if( ssgmGetControlReg(&val) ) {
		ssgmCtrl.ncp_mode = 0xF;
		ssgmCtrl.gen_mode = 0xF;
		ssgmCtrl.sop_source = 0xF;
		ssgmCtrl.mcp_ncp_source = 0xF;
		return -1;
	}

	printf("-------------------------------------------\n");
	printf("Control Reg. Value:\t 0x%08X\n", val);

	ssgmCtrl.ncp_mode = (val >> 1) & 1;
	ssgmCtrl.gen_mode = (val >> 8) & 1;
	ssgmCtrl.sop_source = (val >> 9) & 1;
	ssgmCtrl.mcp_ncp_source = (val >> 16) & 3;

	printf("NCP OP. MODE =\t\t");
	if( ssgmCtrl.ncp_mode == MODE_CONTINOUS) printf("[continuous mode]\n");
	else	printf( "[command mode]\n");

	printf("MCP/NCP GEN. MODE =\t");
	if( ssgmCtrl.gen_mode == GEN_INTERNAL) printf("[internal mode]\n");
	else	printf( "[external mode]\n");

	printf("SOP SOURCE =\t\t");
	if( ssgmCtrl.sop_source == SOP_SRC_MCP ) printf("[MCP]\n");
	else 	printf("[NCP]\n");

	printf("MCP/NCP SOURCE =\t");
	if( ssgmCtrl.mcp_ncp_source == SIG_SRC_LOC)	printf("[local]\n");
	else if(ssgmCtrl.mcp_ncp_source == SIG_SRC_OPT) printf("[optical]\n");
	else if(ssgmCtrl.mcp_ncp_source == SIG_SRC_LVDS) printf("[LVDS]\n");
	else	printf("[unkown]\n");

	printf("-------------------------------------------\n");
	return 0;
}

/*****************************************************************************/
// starts generating NCP
/*****************************************************************************/
int startNcp()
{
	unsigned int val = 0;

	if( ssgmGetControlReg(&val) ) return -1;

	return ssgmSetControlReg( val | (1 << 2));
}

/*****************************************************************************/
// stops generating NCP
/*****************************************************************************/
int stopNcp()
{
	unsigned int val = 0;

	if( ssgmGetControlReg(&val) )  return -1;

	return ssgmSetControlReg(val & ~(1 << 2));
}

/*****************************************************************************/
// END OF ssgm_test.c
/*****************************************************************************/

